#!/bin/bash
# vim: dict+=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   lib.sh of /CoreOS/distribution/Library/sundry
#   Description: Library of functions which are used mainly for parametrization
#   Author: Branislav Blaskovic <bblaskov@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2014 Red Hat, Inc.
#
#   This program is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 2 of
#   the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE.  See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see http://www.gnu.org/licenses/.
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   library-prefix = sundry
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 NAME

distribution/sundry - Library of functions which are used mainly for parametrization

=head1 DESCRIPTION

This library gathers several functions for test case parametrization in RHSCL
and common environments.

=cut

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Variables
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 VARIABLES

Below is the list of global variables. When writing a new library,
please make sure that all global variables start with the library
prefix to prevent collisions with other libraries.

=over

=item sundryEXAMPLE

Example variable, please remove after adding the first variable.

=back

=cut

sundryEXAMPLE="example"

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Functions
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 FUNCTIONS

=head2 sundryDefaultDatabase


    sundryDefaultDatabase

=over

=back

Return default database for current environment.

=cut

sundryDefaultDatabase() {
    if [[ x$DATABASE != xmysql && x$DATABASE != xmariadb ]] ; then
        local out=""
        case "$COLLECTIONS" in
            *mysql*)
                out="mysql"
                ;;
            *mariadb*)
                out="mariadb"
                ;;
            *)
                # java-1.7.0-openjdk is available since RHEL5 so don't
                # test for older releases; mariadb is default since RHEL7
                if rlIsRHEL 5 6 ; then
                    out="mysql"
                else
                    out="mariadb"
                fi
                ;;
        esac
        echo "$out"
    else
        echo "$DATABASE"
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head2 sundryRpmVersionCompare


    sundryRpmVersionCompare RpmName Operator Version

=over

=item RpmName

Name of rpm package to compare

=item Operator

Similar as for test command, one of:
 -gt, -ge, -lt, -le, -eq, -ne

=item Version

Version to compare with. comparison will be constructed as written, eg:

sundryRpmVersionCompare sos -ge 3.2
returns 0 if installed sos version is greater or equal than 3.2

=back

Return 0 if given expression is true.

=cut

sundryRpmVersionCompare() {
    if [ $# -ne 3 ]; then
        rlLogError "Bad number of parameters to sundryRpmVersionCompare()"
        return 2
    fi
    local __evr="$(rpm -q --qf '%{evr}' $1)"
    rlLog "sundryRpmVersionCompare: package $1 nvr is $__evr"
    sundryRpmVersionCompareDirect "$__evr" "$2" "$3"
    return $?
}

true <<'=cut'
=pod

=head2 sundryRpmVersionCompareDirect


    sundryRpmVersionCompareDirect Version1 Operator Version2

=over

=item Version1

First version to compare with.

=item Operator

Similar as for test command, one of:
 -gt, -ge, -lt, -le, -eq, -ne

=item Version2

Second version to compare with. 
Comparison will be constructed as written, eg:

sundryRpmVersionCompareDirect 3.3.1 -ge 3.2
returns 0

=back

Return 0 if given expression is true.

=cut

sundryRpmVersionCompareDirect() {
    if [ $# -ne 3 ]; then
        rlLogError "Bad number of parameters to sundryRpmVersionCompareDirect()"
        return 2
    fi
    rlLog "sundryRpmVersionCompareDirect: $1 $2 $3"
    if [ -x /usr/bin/rpmdev-vercmp ]; then
        rlLog "Comparing rpm version using rpmdev-vercmp tool"
        /usr/bin/rpmdev-vercmp "$1" "$3"
        case "$?" in
            0)  # equals
                if [ "_$2" = "_-eq" -o "_$2" = "_-ge" -o "_$2" = "_-le" ]; then
                    rlLog "sundryRpmVersionCompareDirect: true"
                    return 0
                else
                    rlLog "sundryRpmVersionCompareDirect: false"
                    return 1
                fi
                ;;
            11)  # first version is newer (greater) then second
                if [ "_$2" = "_-ne" -o "_$2" = "_-gt" -o "_$2" = "_-ge" ]; then
                    rlLog "sundryRpmVersionCompareDirect: true"
                    return 0
                else
                    rlLog "sundryRpmVersionCompareDirect: false"
                    return 1
                fi 
                ;;
            12)  # first version is older (lesser) then second
                if [ "_$2" = "_-ne" -o "_$2" = "_-lt" -o "_$2" = "_-le" ]; then
                    rlLog "sundryRpmVersionCompareDirect: true"
                    return 0
                else
                    rlLog "sundryRpmVersionCompareDirect: false"
                    return 1
                fi 
                ;;
            *) # rpmdev-vercmp reports some error
                rlLogError "rpmdev-vercmp error, will try own compare code"
        esac
    fi

    rlLog "Comparing rpm version using own implementation (possibly buggy)"
    rlLogError "sundryRpmVersionCompareDirect: Not implemented yet"
    return 3
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   This is a verification callback which will be called by
#   rlImport after sourcing the library to make sure everything is
#   all right. It makes sense to perform a basic sanity test and
#   check that all required packages are installed. The function
#   should return 0 only when the library is ready to serve.

sundryLibraryLoaded() {
    if true ; then
        rlLogDebug "Library distribution/sundry is loaded."
        return 0
    else
        rlLogError "Loading of distribution/sundry failed."
        return 1
    fi
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Authors
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

true <<'=cut'
=pod

=head1 AUTHORS

=over

=item *

Branislav Blaskovic <bblaskov@redhat.com>

=back

=cut
